/* ppmtocmy3.c - separate a portable pixmap into three portable graymaps
**
** By Eli the Bearded / Benjamin Elijah Griffin, 2005, based on ppmtorgb3.c:
** Copyright (C) 1991 by Jef Poskanzer.
**
** Permission to use, copy, modify, and distribute this software and its
** documentation for any purpose and without fee is hereby granted, provided
** that the above copyright notice appear in all copies and that both that
** copyright notice and this permission notice appear in supporting
** documentation.  This software is provided "as is" without express or
** implied warranty.
*/

#include <string.h>
#include "ppm.h"
#include "pgm.h"

int
main( argc, argv )
    int argc;
    char* argv[];
    {
    FILE* ifp;
    FILE* cynfile;
    FILE* magfile;
    FILE* yelfile;
    const char* basename;
    char filename[100];
    char* cp;
    pixel* pixelrow;
    register pixel* pP;
    gray* grayrow;
    register gray* gP;
    int rows, cols, format, row;
    register int col;
    pixval maxval;


    ppm_init( &argc, argv );

    if ( argc > 2 )
	pm_usage( "[ppmfile]" );

    if ( argc == 2 )
	{
	ifp = pm_openr( argv[1] );
	basename = argv[1];
	cp = strrchr( basename, '.' );
	if ( cp != NULL )
	    *cp = '\0';
	}
    else
	{
	ifp = stdin;
	basename = "noname";
	}

    ppm_readppminit( ifp, &cols, &rows, &maxval, &format );
    pixelrow = ppm_allocrow( cols );
    (void) strcpy( filename, basename );
    (void) strcat( filename, ".cyn" );
    cynfile = pm_openw( filename );
    pgm_writepgminit( cynfile, cols, rows, (gray) maxval, 0 );
    (void) strcpy( filename, basename );
    (void) strcat( filename, ".mag" );
    magfile = pm_openw( filename );
    pgm_writepgminit( magfile, cols, rows, (gray) maxval, 0 );
    (void) strcpy( filename, basename );
    (void) strcat( filename, ".yel" );
    yelfile = pm_openw( filename );
    pgm_writepgminit( yelfile, cols, rows, (gray) maxval, 0 );
    grayrow = pgm_allocrow( cols );

    /* See ppmtocmyk4.c for conversion method comment */
    for ( row = 0; row < rows; ++row )
	{
	ppm_readppmrow( ifp, pixelrow, cols, maxval, format );

	for ( col = 0, pP = pixelrow, gP = grayrow; col < cols;
	      ++col, ++pP, ++gP )
	    *gP = (gray) maxval - (gray) PPM_GETR( *pP );
	pgm_writepgmrow( cynfile, grayrow, cols, maxval, 0 );

	for ( col = 0, pP = pixelrow, gP = grayrow; col < cols;
	      ++col, ++pP, ++gP )
	    *gP = (gray) maxval - (gray) PPM_GETG( *pP );
	pgm_writepgmrow( magfile, grayrow, cols, maxval, 0 );

	for ( col = 0, pP = pixelrow, gP = grayrow; col < cols;
	      ++col, ++pP, ++gP )
	    *gP = (gray) maxval - (gray) PPM_GETB( *pP );
	pgm_writepgmrow( yelfile, grayrow, cols, maxval, 0 );
	}

    pm_close( ifp );
    pm_close( cynfile );
    pm_close( yelfile );
    pm_close( magfile );

    exit( 0 );
    }
